-- Scripted Character Select - v1.0
-- Set up your perfect race!
-- Written by FrivolousAqua
freeslot("sfx_aqpaus") -- Freeslotting the NSMB pause sound

local paused = false -- Is the game """""paused""""" at the moment? (not actually paused)
local scripted = false -- Is the next race scripted?

local scriptedskins = {} -- What skins do we have?
-- Fill up the array with dummy values.
for i = 1, 16 do
	scriptedskins[i] = ""
end

-- This stuff is for the pause menu.
local skintable = {} -- What skins are available?
local currentoption = 0 -- What's the current selection the player left on?
local currentskin = -1 -- What's the current skin the player left on?
local currentplayer = 0 -- What player is the... Player currently viewing?
local selectioncooldown = 0 -- This always decrements. Prevents the selection from going too fast.

-- Whenever you load into a new map, be sure to check if the race is currently scripted.
-- If so, set the characters up properly.
addHook("MapLoad", function()
	paused = false -- Make sure you're no longer paused when the map resets

	-- Loop through all players and assign them the appropriate skin
	for p in players.iterate
		if scripted == true then
			-- Not all slots may have a defined character.
			if scriptedskins[#p + 1] != "" then
				R_SetPlayerSkin(p, scriptedskins[#p + 1])
				p.skincolor = skins[scriptedskins[#p + 1]].prefcolor
				-- 2.4 and above: set the correct name for bots
				if K_SetNameForBot then
					if p.bot then K_SetNameForBot(p, skins[scriptedskins[#p + 1]].realname) end
				end
			end
		end
	end
	
	-- Whenever we load into a map, there might be new characters loaded...
	-- Be sure to add them into the table.
	for s in skins.iterate
		skintable[#s] = s
	end
	
	-- Also, be sure to reset the section cooldown.
	selectioncooldown = 0
end)

-- A lot of the stuff occurs in the ThinkFrame.
-- Mostly just menu processing, honestly.
addHook("ThinkFrame", function()
	-- I cannot use a PlayerThink for this, as hitlag disables the player thinker.
	for p in players.iterate
		-- You can only use this if you're the host.
		if #p != 0 then continue end
	
		-- This is only called if the player hits the vote button.
		-- Since this mod is intended for singleplayer, this is no issue.
		if (p.cmd.buttons & 1<<7) and not (p.oldcmd.buttons & 1<<7) then
			if paused == false then
				S_StartSound(nil, sfx_aqpaus)
				paused = true
			else
				S_StartSound(nil, sfx_aqpaus)
				paused = false
			end
		end
	
		-- If the game is paused, all players will be permanently stuck in place.
		-- Not my fault if they get hit by some projectile or something.
		if paused == true then
			p.mo.hitlag = 1
		end
		
		-- Paused input logic
		if paused then
			-- Going down the menu
			if (p.cmd.throwdir < 0) and not (p.oldcmd.throwdir < 0) then
				S_StartSound(nil, sfx_menu1)
				currentoption = $ + 1
			-- Going up the menu
			elseif (p.cmd.throwdir > 0) and not (p.oldcmd.throwdir > 0)
				S_StartSound(nil, sfx_menu1)
				currentoption = $ - 1
			end
			
			-- There's only 3 options available; if it goes below 0 or over 2 loop around
			if currentoption < 0 then currentoption = 2 end
			if currentoption > 2 then currentoption = 0 end
			
			-- If we're on option 0, select whether or not the next race is scripted
			if currentoption == 0 then
				if (p.cmd.turning != 0) and not (p.oldcmd.turning != 0) then
					S_StartSound(nil, sfx_menu1)
					if scripted == true then
						scripted = false
					else
						scripted = true
					end
				-- Tabbing left
				end
			-- If we're on option 1, select which player to edit
			-- It'll change the current skin appropriately as well to match the player slot
			elseif currentoption == 1 then
				-- Tabbing to either side pretty much does the same thing.
				-- Just, well, in either order.
				-- There probably was a better way to write this, like a function or something.
			
				-- Tabbing right
				if (p.cmd.turning < 0) and not (p.oldcmd.turning < 0) then
					S_StartSound(nil, sfx_menu1)
					currentplayer = $ + 1
					
					-- There's no extra player slots beyond 16 players.
					if currentplayer < 0 then currentplayer = 15 end
					if currentplayer > 15 then currentplayer = 0 end
				
					if scriptedskins[currentplayer + 1] == "" then
						currentskin = -1
					else
						currentskin = #skins[scriptedskins[currentplayer + 1]]
					end
				-- Tabbing left
				elseif (p.cmd.turning > 0) and not (p.oldcmd.turning > 0)
					S_StartSound(nil, sfx_menu1)
					currentplayer = $ - 1
					
					-- There's no extra player slots beyond 16 players.
					if currentplayer < 0 then currentplayer = 15 end
					if currentplayer > 15 then currentplayer = 0 end

					if scriptedskins[currentplayer + 1] == "" then
						currentskin = -1
					else
						currentskin = #skins[scriptedskins[currentplayer + 1]]
					end
				end
			-- If we're on option 2, this is the fun part: actually setting the character
			elseif currentoption == 2 then
				-- Similar case to the last option, but this time the character's being chosen.
				-- It only cycles through valid characters.
				-- If the player continuously holds a direction, it'll go faster.
			
				-- Tabbing right
				if (p.cmd.turning < 0) and selectioncooldown == 0 then
					S_StartSound(nil, sfx_menu1)
					currentskin = $ + 1
					if (p.cmd.turning < 0) and not (p.oldcmd.turning < 0) then
						selectioncooldown = TICRATE / 2
					else
						selectioncooldown = 3
					end
				-- Tabbing left
				elseif (p.cmd.turning > 0) and selectioncooldown == 0 then
					S_StartSound(nil, sfx_menu1)
					currentskin = $ - 1
					if (p.cmd.turning > 0) and not (p.oldcmd.turning > 0) then
						selectioncooldown = TICRATE / 2
					else
						selectioncooldown = 3
					end
				end
				
				-- If the player isn't holding it at all, no point in the cooldown; just set it to 0
				if p.cmd.turning == 0 then
					selectioncooldown = 0
				end
				
				-- If you press brake, however, it'll default to a random character (no skin).
				if (p.cmd.buttons & BT_BRAKE) and not (p.oldcmd.buttons & BT_BRAKE) then
					S_StartSound(nil, sfx_s3k7b)
					currentskin = -1
				end
				
				-- Don't select stuff that doesn't exist!
				if currentskin < -1 then currentskin = #skintable end
				if currentskin > #skintable then currentskin = -1 end
				
				if currentskin == -1 then
					scriptedskins[currentplayer + 1] = ""
				else
					scriptedskins[currentplayer + 1] = skintable[currentskin].name
				end
			end
		end
	end
	
	-- this always counts down
	selectioncooldown = max(0, $ - 1)
end)

--
-- HUD STUFF
--

-- Just for the character icons.
local currenticon = nil
local currentcolormap = nil

local function selectionscreen(v, p)
	if paused == false then return end
	
	-- Draw a filter over the screen for some clarity.
	v.drawFill(0, 0, 320, 200, 31 | V_50TRANS)
	
	-- Some info as to what the screen actually is
	v.drawString(160, 10, "Scripted Character Select - v1.0", V_SNAPTOTOP, "thin-center")
	v.drawString(160, 20, "Set up your perfect race!", V_SNAPTOTOP, "thin-center")
	
	-- hehe
	v.drawString(160, 190, "Written by FrivolousAqua", V_SNAPTOBOTTOM, "thin-center")

	-- Well? Is it?
	local scriptedtext = "Make the next race scripted? "
	if currentoption == 0 then scriptedtext = "\130" + $ end
	v.drawString(160, 60, scriptedtext .. tostring(scripted), V_SNAPTOTOP | V_SNAPTOBOTTOM, "thin-center")
	
	-- Which player are we currently editing?
	local playertext = "Currently editing player: "
	if currentoption == 1 then playertext = "\130" + $ end
	v.drawString(160, 70, playertext .. tostring(currentplayer), V_SNAPTOTOP | V_SNAPTOBOTTOM, "thin-center")
	
	-- This is for the displayed skin names.
	local skinnametext = ""
	local skinnameinternal = ""
	-- If you're not on a character (the random button), say so.
	if currentskin == -1 then
		skinnametext = "Random"
		skinnameinternal = "(No skin selected!)"
		currenticon = v.cachePatch("M_NORANK")
	else
		-- Otherwise, just fetch the character names from the skin table
		skinnametext = skintable[currentskin].realname
		skinnameinternal = skintable[currentskin].name
		-- Also update the icon and colormap accordingly
		currenticon = v.getSprite2Patch(skintable[currentskin].name, SPR2_XTRA, false, 0)
		currentcolormap = v.getColormap(skintable[currentskin].name, skintable[currentskin].prefcolor)
	end
	if currentoption == 2 then skinnametext = "\130" + $ end
	v.draw(117, 82, currenticon,  V_SNAPTOTOP | V_SNAPTOBOTTOM, currentcolormap)
	v.drawString(134, 80, skinnametext, V_SNAPTOTOP | V_SNAPTOBOTTOM, "thin")
	v.drawString(134, 91, skinnameinternal, V_SNAPTOTOP | V_SNAPTOBOTTOM, "thin")
end

hud.add(selectionscreen, "game")
